#include <windows.h>

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <commctrl.h>
#include "..\hrdl.h"
#include "resource.h"

#define ID_TIMER 1
#define MAX_COLUMNS 2
#define WM_REFRESH WM_USER+1

#define MAX_READINGS 10000
#define MAX_DEVICE_READINGS 100 


HINSTANCE hInstance;

long CALLBACK WndProc (HWND, UINT, UINT, LONG);

typedef struct tSettings
{
	long noOfSamples;
	HRDL_CONVERSION_TIME conversionTime;
	long timeInterval;
} SETTINGS;

typedef struct tChannel
{
	short		enabled;
	short		range;
	int			singleEnded;
	float		readings[MAX_READINGS];
	short		nextReadingPosition;
} CHANNEL;

typedef struct tUnit
{
	CHANNEL channels [HRDL_MAX_ANALOG_CHANNELS + 1];
	int				firstRange;
	int				lastRange;
	int				noOfChannels;
	SETTINGS	settings;
	short			handle;
	BOOL			running;
} UNIT;

UNIT g_unit;
short g_nEnabledChannels;
int g_height, g_width;
float g_gain[] = {1.f, 2.f, 4.f, 8.f, 16.f, 32.f, 64.f};
BOOL g_stopSampling;


#define IN_RANGE_INCL(input, lower, upper) (input>=lower && input<=upper)

float ADC2mV(short channel, long sample)
{
	long max, min;

	HRDLGetMinMaxAdcCounts(g_unit.handle, &min, &max, channel);

	return ((float)sample * (float) 2500 / g_gain[g_unit.channels[channel].range] / (float)max);
}

BOOL SetUnitRunning(HWND hwnd)
{
	BOOL translated;
	short conversionTime[] = {60, 100, 180, 340, 660};
	int i;

	// gather new settings information
	g_unit.settings.conversionTime = (HRDL_CONVERSION_TIME)SendDlgItemMessage(hwnd, IDC_CONVERSION_TIME, CB_GETCURSEL, 0, 0);
  
	g_unit.settings.timeInterval = GetDlgItemInt(hwnd, IDC_TIME_INTERVAL, &translated, TRUE);
	if(!IN_RANGE_INCL(g_unit.settings.timeInterval, 1, 60000))
	{
		MessageBox(hwnd, "Time Interval not valid", "Set up unit", MB_OK);
		return FALSE;
	}

	g_unit.settings.noOfSamples = GetDlgItemInt(hwnd, IDC_NO_OF_SAMPLES, &translated, TRUE);
	// this has been restricted to 10000 for this example it can be any number of samples 
	if(!IN_RANGE_INCL(g_unit.settings.noOfSamples, 1, MAX_READINGS))
	{
    MessageBox(hwnd, "No of sample out of range", "Set up unit", MB_OK);
		return FALSE;
	}

	for(i = 1; i <= g_unit.noOfChannels; i++)
	{
		g_unit.channels[i].nextReadingPosition = 0;
		if(!HRDLSetAnalogInChannel(g_unit.handle,
													 i,
													 g_unit.channels[i].enabled,
													 g_unit.channels[i].range,
													 g_unit.channels[i].singleEnded))
		{
      MessageBox(hwnd, "Analogue channels failed", "Set up unit", MB_OK);
		  return FALSE;
		}
	}

	HRDLGetNumberOfEnabledChannels(g_unit.handle, &g_nEnabledChannels);

	if(g_nEnabledChannels == 0)
	{
    MessageBox(hwnd, "No channels enabled", "Set up unit", MB_OK);
		return FALSE;
	}

	if(!IN_RANGE_INCL(g_nEnabledChannels, 0, g_unit.noOfChannels))
	{
    MessageBox(hwnd, "Number of channels invalid", "Set up unit", MB_OK);
		return FALSE;
	}

	if(g_unit.settings.timeInterval <= g_nEnabledChannels * conversionTime[g_unit.settings.conversionTime])
	{
    MessageBox(hwnd, "Not enough time between samples", "Set up unit", MB_OK);
		return FALSE;
	}

	if(!HRDLSetInterval(g_unit.handle, g_unit.settings.timeInterval, (short)g_unit.settings.conversionTime))
	{
	  MessageBox(hwnd, "Set Interval failed", "Set up unit", MB_OK);
		return FALSE;
	}
	
	if(!HRDLRun(g_unit.handle, MAX_DEVICE_READINGS, (short)HRDL_BM_STREAM))
	{
	  MessageBox(hwnd, "Failed to start", "Start streaming", MB_OK);
		return FALSE;
	}

	return TRUE;
}

/****************************************************************************
 *
 *
 ****************************************************************************/

long CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	char *szConversion[] = {"60ms", "100ms", "180ms", "340ms", "660ms"};
  char *szVoltage[] = {"2500mV", "1250mV", "625mV", "313mV", "156mV", "78mV", "39mV"};
	char *szColumnTitle[] = {"Single Ended", "Channel"};
	int i;
	LVCOLUMN column; 
	LVITEM lvi;
	char str[256];
	static BOOL initial;
	RECT rect;
  int columnWidth;
	int iRowCount;
	char line[80];
	long samples[MAX_DEVICE_READINGS];
	long numberofDeviceReadings;
	short overflow;
	short channel;
	float voltagePerPixel;
	HPEN 	hpen, oldPen;
	float pointY, pointX;
	int position;
	int selectionMark;
	unsigned int numberSelected;
	int voltageRange;
	int maxRange;
	unsigned long pencolour[HRDL_MAX_ANALOG_CHANNELS + 1] = {0xFF0000, 0x00FF00, 0x0000FF, 0xFFFF00,
																													 0xFF00FF, 0x00FFFF, 0x110000, 0x001100,
																													 0x000011, 0x111100, 0x110011, 0x001111,
																													 0x770000, 0x007700, 0x000077, 0x777700};

  switch (message)
  {
	  case WM_CREATE:
		g_height = 235;
		g_width = 366;
		g_nEnabledChannels = 0;
    initial = FALSE;
		g_unit.handle = HRDLOpenUnit();
		if(g_unit.handle >0)
		{
			HRDLGetUnitInfo(g_unit.handle, line, 80, HRDL_VARIANT_INFO);
			switch(atoi(line))
			{
			case 20:
				g_unit.firstRange = HRDL_2500_MV;
				g_unit.lastRange = HRDL_1250_MV;
				g_unit.noOfChannels = 8;

			break;

			case 24:
				g_unit.firstRange = HRDL_2500_MV;
				g_unit.lastRange = HRDL_39_MV;
				g_unit.noOfChannels = 16;
			break;

			default:
        sprintf(line, "No unit Found");
			  SendMessage(hwnd, WM_SETTEXT, 0, (LPARAM) line);
			}
		}
		g_unit.settings.conversionTime = HRDL_60MS;
	  g_unit.settings.noOfSamples = 500;
 	  g_unit.settings.timeInterval = 1000;
		g_unit.running = FALSE;
		SetTimer(hwnd, ID_TIMER, 100, NULL);

    break;

		case WM_REFRESH:

      HRDLGetUnitInfo(g_unit.handle, line, 80, HRDL_VARIANT_INFO);
			sprintf(str, "ADC-%d", atoi(line));
			SendMessage(hwnd, WM_SETTEXT, 0, (LPARAM) str);

			for (i = 0; i <= g_unit.lastRange; i++)
			{
        SendDlgItemMessage(hwnd, IDC_VOLTAGE_RANGE, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)szVoltage[i]);
			}
			SendDlgItemMessage(hwnd, IDC_VOLTAGE_RANGE, CB_SETCURSEL, 0, 0);

      for (i = 0; i < HRDL_MAX_RANGES; i++)
			{
				SendDlgItemMessage(hwnd, IDC_CONVERSION_TIME, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)szConversion[i]);
			}
			SendDlgItemMessage(hwnd, IDC_CONVERSION_TIME, CB_SETCURSEL, 0, 0);

			SetDlgItemInt(hwnd, IDC_NO_OF_SAMPLES, 500, TRUE); 
			SetDlgItemInt(hwnd, IDC_TIME_INTERVAL, 61, TRUE);

			GetClientRect(GetDlgItem (hwnd, IDC_CHANNELS), &rect);
			columnWidth = (rect.right - rect.left) / MAX_COLUMNS;
			SendDlgItemMessage(hwnd, IDC_CHANNELS, LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM) (DWORD) (LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES), (LPARAM) (DWORD)(LVS_EX_CHECKBOXES |LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES));

			column.mask = LVCF_WIDTH;
			column.cx = 0;
			column.fmt =  0;
			column.pszText = "";
			column.cchTextMax = (int)strlen("");
			column.iSubItem = 0;

			SendDlgItemMessage(hwnd, IDC_CHANNELS, LVM_INSERTCOLUMN,(WPARAM) 0, (LPARAM) (LV_COLUMN*) &column);
			column.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_FMT | LVCF_SUBITEM;
			for (i = 0; i < MAX_COLUMNS;i++)
			{
				ZeroMemory(str, sizeof(str));
				sprintf(str, "%s", szColumnTitle[i]);
	      

				if(i < MAX_COLUMNS-1)
				{
					column.cx = columnWidth;
				}
				else
				{
					column.cx = rect.right - (columnWidth * (MAX_COLUMNS-1));
				}
				column.fmt =  LVCFMT_CENTER;
				column.pszText = str;
				column.cchTextMax = (int)strlen(str);
				column.iSubItem = 0;

				SendDlgItemMessage(hwnd, IDC_CHANNELS, LVM_INSERTCOLUMN,(WPARAM) i+1, (LPARAM) (LV_COLUMN*) &column);
			}
			SendDlgItemMessage(hwnd, IDC_CHANNELS, LVM_DELETECOLUMN,(WPARAM) 0, (LPARAM) 0);

						// insert the data into the listview
			lvi.mask = LVIF_TEXT;
			lvi.iSubItem = 0;
			iRowCount = 0;
			for (i = 0; i < g_unit.noOfChannels;i++)
			{
				lvi.cchTextMax = sizeof(str);
  			lvi.pszText = str;

  			lvi.iItem = i;
				lvi.iSubItem = iRowCount++;
				if((BYTE)(i+1) & 0x01)
				{
					sprintf(str, "");
				}
				else
				{
					sprintf(str, "Always");
				}
				ListView_InsertItem(GetDlgItem(hwnd, IDC_CHANNELS), &lvi);

				ListView_SetCheckState(GetDlgItem(hwnd, IDC_CHANNELS), i,TRUE);

				lvi.iSubItem = iRowCount;
				sprintf(str, "Channel %i - Available", i + 1);
				ListView_SetItemText(GetDlgItem(hwnd, IDC_CHANNELS), i, 1, str);
				iRowCount = 0;
			}
		break;

    case WM_PAINT:
      hdc = BeginPaint ( hwnd, &ps );

      Rectangle(hdc, 243, 0, 601, 235);
			maxRange = HRDL_MAX_RANGES;
      for (i = HRDL_ANALOG_IN_CHANNEL_1; i<=g_unit.noOfChannels;i++)
			{
				if(g_unit.channels[i].enabled && maxRange > g_unit.channels[i].range)
				{
					maxRange = g_unit.channels[i].range;
				}
			}

			voltagePerPixel = ((float)(g_height - 2) / 2.f) /(2500.f / g_gain[maxRange]);
			// only draw the results if two or more readings are availalble
			for (i = HRDL_ANALOG_IN_CHANNEL_1; i<=g_unit.noOfChannels;i++)
			{
				if(g_unit.channels[i].enabled && g_unit.channels[i].nextReadingPosition > 1)
				{
					// draw the results for this channel
					hpen = CreatePen ( PS_SOLID , 0, pencolour[i]);
          oldPen = SelectObject ( hdc, hpen );
					if(g_unit.channels[i].nextReadingPosition > g_width)
					{
					  position = g_unit.channels[i].nextReadingPosition - g_width;
					}
					else
					{
						position = 0;
					}
					// first point
					pointY = (g_height / 2) - ((voltagePerPixel * g_unit.channels[i].readings[position]));
					position++;
					pointX = 243;
          MoveToEx (hdc, (int)pointX, (int)pointY , (LPPOINT) NULL);
					while(position < g_unit.channels[i].nextReadingPosition)
					{
					  pointY = (g_height / 2) - ((voltagePerPixel * g_unit.channels[i].readings[position]));
						position++;
						pointX++;
						LineTo ( hdc, (int)pointX, (int)pointY );						
            MoveToEx (hdc, (int)pointX, (int)pointY, (LPPOINT) NULL);
					}
				}
			}

      EndPaint(hwnd, &ps);

			if(!initial)
			{
			  SendMessage(hwnd, WM_REFRESH, 0, 0);
				initial = TRUE;
	    }
    break;

    case WM_TIMER:
		 if(g_unit.running)
		 {
			 if(!HRDLReady(g_unit.handle))
			 {
					break;
			 }
			 numberofDeviceReadings = HRDLGetValues(g_unit.handle, samples, &overflow, MAX_DEVICE_READINGS/g_nEnabledChannels);
			 for(i = 0; i <  numberofDeviceReadings * g_nEnabledChannels; )
			 {
				 for(channel = HRDL_ANALOG_IN_CHANNEL_1; channel <= g_unit.noOfChannels; channel++)
				 {
					 if(g_unit.channels[channel].enabled)
					 {
						 g_unit.channels[channel].readings[g_unit.channels[channel].nextReadingPosition] = ADC2mV(channel, samples[i]);
						 g_unit.channels[channel].nextReadingPosition++;
						 i++;
					 }
					 if(g_unit.channels[channel].nextReadingPosition > g_unit.settings.noOfSamples)
					 {
						 g_stopSampling = TRUE;
					 }
				 }
			 }
			  
			 if(g_stopSampling)
			 {
				 SendMessage(hwnd, WM_COMMAND, IDC_STOP, 0);
			 }
			 if(numberofDeviceReadings)
			 {
				 rect.left = 243;
         rect.right = 243 + g_width;
         rect.top = 0;
         rect.bottom = g_height;
				 InvalidateRect(hwnd, &rect, TRUE);
			 }
		 }
    break;

    case WM_COMMAND:
    switch ( wParam )
    {
      case IDC_SET_RANGE:
				if(g_unit.running)
				{
					MessageBox(hwnd, "Not avalable whilst sampling", "Set Channels", MB_OK);
					break;
				}
				selectionMark =  0;
				numberSelected =  ListView_GetSelectedCount(GetDlgItem(hwnd, IDC_CHANNELS));
				voltageRange = (int) SendDlgItemMessage(hwnd, IDC_VOLTAGE_RANGE, CB_GETCURSEL, 0, 0);
	      
				while (selectionMark < g_unit.noOfChannels && numberSelected > 0)
				{
					if(ListView_GetItemState(GetDlgItem(hwnd, IDC_CHANNELS), selectionMark, LVIS_SELECTED) == LVIS_SELECTED)
					{
						if((selectionMark & 0x01) && g_unit.channels[(selectionMark + 1) - 1].enabled &&  !g_unit.channels[(selectionMark + 1) - 1].singleEnded )
						{
						  g_unit.channels[selectionMark + 1].enabled = FALSE;
							ListView_SetItemText(GetDlgItem(hwnd, IDC_CHANNELS), selectionMark, 1, "Not Available");
						}
						else
						{
						  g_unit.channels[selectionMark + 1].enabled = TRUE;
						}
						g_unit.channels[selectionMark + 1].singleEnded  = ListView_GetCheckState(GetDlgItem(hwnd, IDC_CHANNELS), selectionMark);
						if(!(selectionMark & 0x01) &&  !g_unit.channels[selectionMark + 1].singleEnded)
						{
							g_unit.channels[selectionMark + 1 + 1].enabled = FALSE;
							ListView_SetItemText(GetDlgItem(hwnd, IDC_CHANNELS), selectionMark + 1, 1, "Not Available");
						}
						g_unit.channels[selectionMark + 1].range = (short) voltageRange;
						if(g_unit.channels[selectionMark + 1].enabled)
						{
						  sprintf(str, "Channel %d - %s", selectionMark +1, szVoltage[voltageRange]);
						  ListView_SetItemText(GetDlgItem(hwnd, IDC_CHANNELS), selectionMark, 1, str);
						}
						ListView_SetItemState(GetDlgItem(hwnd, IDC_CHANNELS), selectionMark, !LVIS_SELECTED, LVIS_SELECTED);
            numberSelected--;
					}
					selectionMark++;
				}
      break;

			case IDC_RESET:
 			  if(g_unit.running)
				{
					MessageBox(hwnd, "Not avalable whilst sampling", "Set Channels", MB_OK);
					break;
				}

				numberSelected =  ListView_GetSelectedCount(GetDlgItem(hwnd, IDC_CHANNELS));
	      selectionMark = 0;
				while (selectionMark < g_unit.noOfChannels && numberSelected > 0)
				{
					if(ListView_GetItemState(GetDlgItem(hwnd, IDC_CHANNELS), selectionMark, LVIS_SELECTED) == LVIS_SELECTED)
					{
						if(!(selectionMark & 0x01) && !g_unit.channels[selectionMark + 1].singleEnded)
						{
							sprintf(str, "Channel %d - Available", selectionMark +2);
						  ListView_SetItemText(GetDlgItem(hwnd, IDC_CHANNELS), selectionMark + 1, 1, str);
						}
						g_unit.channels[selectionMark + 1].enabled = FALSE;
						g_unit.channels[selectionMark + 1].singleEnded = 1;
						g_unit.channels[selectionMark + 1].range = 0;
						ListView_SetCheckState(GetDlgItem(hwnd, IDC_CHANNELS), selectionMark, TRUE);
						sprintf(str, "Channel %d - Available", selectionMark +1);
						ListView_SetItemText(GetDlgItem(hwnd, IDC_CHANNELS), selectionMark, 1, str);
						ListView_SetItemState(GetDlgItem(hwnd, IDC_CHANNELS), selectionMark, !LVIS_SELECTED, LVIS_SELECTED);
            numberSelected--;
					}
					selectionMark++;
				}

			break;

      case IDC_STOP:
				if(g_unit.running)
				{
					g_unit.running = FALSE;
					HRDLStop(g_unit.handle);
					SetDlgItemText(hwnd, IDC_MESSAGES, "Stopped Sampling");
				}
      break;

      case IDC_START:
				if(g_unit.handle && !g_unit.running)
				{
					if(SetUnitRunning(hwnd))
					{
						SetDlgItemText(hwnd, IDC_MESSAGES, "Started Sampling"); 
						g_stopSampling = FALSE;
					  g_unit.running = TRUE;
					}
				}
      break;

    }
    break;

    case WM_DESTROY:
			if(g_unit.running = TRUE)
				HRDLStop(g_unit.handle);
      KillTimer(hwnd, ID_TIMER);
      PostQuitMessage (0);
    return 0;

  }
  return (long)DefWindowProc ( hwnd, message, wParam, lParam );
}

/****************************************************************************
 *
 *
 ****************************************************************************/

int CALLBACK WinMain (
  HINSTANCE hInst,
  HINSTANCE hPrevInstance,
  LPSTR lpszCmdLine,
  int nCmdShow )

{
  static char 	szClass [] = "HRDL";
  MSG         	msg;
  WNDCLASS    	wndclass;
  INITCOMMONCONTROLSEX  CommControls;
	HWND hwnd;

  hInstance = hInst;

  if ( !hPrevInstance )
  {
    wndclass.style          = CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc    = (WNDPROC) WndProc;
    wndclass.cbClsExtra     = 0;
    wndclass.cbWndExtra     = DLGWINDOWEXTRA;
    wndclass.hInstance      = hInstance;
    wndclass.hIcon          = NULL;
    wndclass.hCursor        = LoadCursor (NULL, IDC_ARROW);
    wndclass.hbrBackground  = (HBRUSH) (COLOR_WINDOW + 1);
    wndclass.lpszMenuName   = NULL;
    wndclass.lpszClassName  = szClass;
    RegisterClass (&wndclass);
  }

	CommControls.dwICC = ICC_LISTVIEW_CLASSES | ICC_BAR_CLASSES | ICC_ANIMATE_CLASS;
  CommControls.dwSize = sizeof(INITCOMMONCONTROLSEX);
  InitCommonControlsEx(&CommControls);

  hwnd = CreateDialog ( hInstance, szClass, 0, NULL ) ;

  ShowWindow (hwnd, nCmdShow);
  UpdateWindow(hwnd);

  while ( GetMessage ( &msg, NULL, 0, 0 ) )
  {
    TranslateMessage ( &msg );
    DispatchMessage ( &msg );
  }

  return (int) msg.wParam ;
}